import * as d3 from 'd3';

function renderData(dataElements) {

    let width = 700;
    let height = 500;
    let offset = 200;

    // lat: x, long: y, dock_count: size, city: color
    let lat = dataElements.map(o => +o.lat);
    let long = dataElements.map(o => +o.long);
    let city = new Set(dataElements.map(o => o.city));
    let dataDict = {};

    let predata = dataElements.map(o => ({
        lat: +o.lat,
        long: +o.long,
        dock_count: +o.dock_count,
        city: o.city
    }));

    // color scale
    const color = d3.scaleOrdinal()
        .domain(city)
        .range(d3.schemeCategory10);

    predata.forEach(element => {
        if (!dataDict[element.city]) {
            // [count, lat, lng, dock_count summ]
            dataDict[element.city] = [0, 0, 0, 0];
        }
        let temp = dataDict[element.city];
        temp[0] += 1;
        temp[1] += element.lat;
        temp[2] += element.long;
        temp[3] += element.dock_count;
        dataDict[element.city] = temp;
    });

    let data = [];
    let dock_values = [];
    let max_dock_count = 0;
    for (let [key, value] of Object.entries(dataDict)) {
        let temp = {};
        temp.city = key;
        temp.lat = value[1] / value[0];
        temp.long = value[2] / value[0];
        temp.dock_count = value[3];
        if (max_dock_count < temp.dock_count) {
            max_dock_count = temp.dock_count;
        }
        dock_values.push(temp.dock_count);
        data.push(temp);
    };

    let margin = ({ top: 50, right: 60, bottom: 40, left: 75 });

    let x = d3.scaleLinear()
        .domain(d3.extent(long)).nice()
        .range([margin.left, width - margin.right]);

    let y = d3.scaleLinear()
        .domain(d3.extent(lat)).nice()
        .range([height - margin.bottom, margin.top]);

    let z = d3.scaleLinear()
        .domain([0, max_dock_count])
        .range([1, 45]);

    let xAxis = g => g
        .attr("transform", `translate(0,${height - margin.bottom})`)
        .call(d3.axisBottom(x));

    let yAxis = g => g
        .attr("transform", `translate(${margin.left},0)`)
        .call(d3.axisLeft(y));

    const svg = d3.select("body").append("svg")
        .attr("width", width)
        .attr("height", height);

    svg.append("g")
        .call(xAxis);

    svg.append("g")
        .call(yAxis);

    svg.append("g")
        .selectAll("circle")
        .data(data)
        .join("circle")
        .attr("fill", color)
        .attr("cx", d => x(d.long))
        .attr("cy", d => y(d.lat))
        .attr("r", d => z(d.dock_count));

    svg.append("g")
        .selectAll("circle")
        .data(data)
        .join("line")
        .attr("x1", d => x(d.long))
        .attr("x2", d => x(d.long) + 50)
        .attr("y1", d => y(d.lat))
        .attr("y2", d => y(d.lat))
        .attr("stroke", "gray")
        .style("stroke-dasharray", ("3,2"));

    svg.append("g")
        .selectAll("circle")
        .data(data)
        .join("text")
        .attr("x", d => x(d.long) + 52)
        .attr("y", d => y(d.lat))
        .text(d => d.dock_count + " Bikes")
        .style("font-size", 12)
        .attr("alignment-baseline", "middle");

    svg.append("g")
        .selectAll("circle")
        .data(data)
        .join("text")
        .attr("x", d => x(d.long) + 52)
        .attr("y", d => y(d.lat) + 10)
        .text(d => d.city)
        .style("font-size", 12)
        .attr("alignment-baseline", "middle");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width - 350)
        .attr("y", height - 3)
        .style("font-size", "14px")
        .text("Longitude");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "150")
        .attr("y", "-10")
        .attr("transform", "translate(30, 400) rotate(270)")
        .style("font-size", "14px")
        .text("Latitude");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width - 450)
        .attr("y", "20")
        .style("font-size", "18px")
        .text("Location of Bike Stations by City in Bay Area.");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width - 450)
        .attr("y", "35")
        .style("font-size", "12px")
        .text("2013 year.");
};

d3.csv(
    "https://gist.githubusercontent.com/olga-kondr/64dc8df90513ada23e183cbb9fa1f9d4/raw/a69aa9c03948a74bb7453bdbcc7a3c70d81d9d9e/weather.csv"
).then(data => renderData(data));